<?php

namespace Drupal\config_warning\Hook;

use Drupal\block\BlockListBuilder;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\user\Form\UserPermissionsForm;

/**
 * Defines form hooks for the config_warning module.
 */
class FormHooks {
  use StringTranslationTrait;

  /**
   * Constructs a new FormHooks instance.
   *
   * @param \Drupal\Core\Routing\AdminContext $adminContext
   *   The admin context service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    private readonly AdminContext $adminContext,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly MessengerInterface $messenger,
  ) {}

  /**
   * Adds a warning message if the altered form modifies configuration.
   *
   * @param array $form
   *   The form render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state object.
   * @param string $form_id
   *   ID of the form to check.
   */
  #[Hook('form_alter')]
  public function formAlter(array &$form, FormStateInterface $form_state, string $form_id): void {
    // Skip if this is not an admin route.
    if (!$this->adminContext->isAdminRoute()) {
      return;
    }

    $config = $this->configFactory->get('config_warning.settings');

    // Skip if warnings are disabled.
    if (!$config->get('enabled')) {
      return;
    }

    $form_object = $form_state->getFormObject();
    $form_alters_config = FALSE;

    // Check for forms implementing getEditableConfigNames().
    if (method_exists($form_object, 'getEditableConfigNames')) {
      $form_alters_config = TRUE;
    }

    // Check if editing a config entity.
    if ($form_object instanceof EntityForm) {
      $entity = $form_object->getEntity();
      if ($entity instanceof ConfigEntityInterface && !$entity->isNew()) {
        $form_alters_config = TRUE;
      }
    }

    // User permissions form modifies configuration.
    if ($form_object instanceof UserPermissionsForm) {
      $form_alters_config = TRUE;
    }

    // Block listing form modifies configuration.
    if ($form_object instanceof BlockListBuilder) {
      $form_alters_config = TRUE;
    }

    // Display the warning message if the form alters configuration.
    if ($form_alters_config) {
      $this->messenger->addWarning($this->t($config->get('warning_message')));
    }
  }

}
